#include "stdafx.h"
#include "Drawing.h"
#include "DrawingDoc.h"
#include "PenDlg.h"
#include "ArrayView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CArrayView, CView)

CArrayView::CArrayView(){
	CurSel=-1;
	pLine=0;	pList=0;
	nPoints=0;	nItems=0;
}

CArrayView::~CArrayView(){
	delete pList;
}

BEGIN_MESSAGE_MAP(CArrayView, CView)
	//{{AFX_MSG_MAP(CArrayView)
	ON_WM_CONTEXTMENU()
	ON_COMMAND(ID_LINE_SIZE, OnLineSize)
	ON_COMMAND(ID_LINE_POINTS, OnLinePoints)
	ON_COMMAND(ID_LINE_DEL, OnLineDel)
	ON_COMMAND(ID_LINE_COLOR, OnLineColor)
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CArrayView::OnDraw(CDC* pDC) {	}

#ifdef _DEBUG
void CArrayView::AssertValid() const{	CView::AssertValid(); }

void CArrayView::Dump(CDumpContext& dc) const {	CView::Dump(dc);}

CDrawingDoc* CArrayView::GetDocument() {
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CDrawingDoc)));
	return (CDrawingDoc*)m_pDocument;
}
#endif

void CArrayView::OnInitialUpdate() {
	pList = new CListCtrl;
	CRect r(40,20,350,155);
	pList->Create(WS_VISIBLE | WS_THICKFRAME | LVS_SHOWSELALWAYS 
				| LVS_REPORT | LVS_SINGLESEL,r,this,IDC_DRAWLIST);
	pList->InsertColumn (0,"Line",LVCFMT_CENTER,40,0);
	pList->InsertColumn (1,"Width",LVCFMT_CENTER,50,0);
	pList->InsertColumn (2,"Red",LVCFMT_CENTER,50,0);
	pList->InsertColumn (3,"Green",LVCFMT_CENTER,50,0);
	pList->InsertColumn (4,"Blue",LVCFMT_CENTER,50,0);
	pList->InsertColumn (5,"Points",LVCFMT_CENTER,50,0);
	CView::OnInitialUpdate();
}

void CArrayView::GetLine () {
	PenSize = pLine->PenSize;
	Color = pLine->Color;
	Red   = GetRValue(Color);
	Green = GetGValue(Color);
	Blue  = GetBValue(Color);
	nPoints = pLine->Points.GetSize();
}

void CArrayView::SetLine (int n) {
	GetLine();
	CString s[6];
	s[0].Format("%d",n+1);
	s[1].Format("%d",PenSize);
	s[2].Format("%d",Red);
	s[3].Format("%d",Green);
	s[4].Format("%d",Blue);
	s[5].Format("%d",nPoints);
	for (int i=0; i<6; i++)
		pList->SetItemText (n,i,s[i]);
}

POSITION CArrayView::SelectItem(TFind find) {
	if (find==Find) {
		CurSel=pList->GetNextItem (-1,LVNI_SELECTED);
		if (CurSel < 0) CurSel = 0;
	}
	pList->SetItem (CurSel,0,LVIF_STATE,0,0,LVIS_SELECTED,LVIS_SELECTED,0);
	pList->SetItem (CurSel,0,LVIF_STATE,0,0,LVIS_FOCUSED,LVIS_FOCUSED,0);
	pList->EnsureVisible (CurSel,false);
	POSITION pos;
	if (! (pos=pDoc->Sketch.FindIndex(CurSel))) 
			pLine = NULL;
	else	pLine = pDoc->Sketch.GetAt(pos);
	if (pLine) 	GetLine();
	return pos;
}

void CArrayView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) {
	pDoc = GetDocument();
	nItems = pDoc->Sketch.GetCount();
	switch (lHint){
	case UPD_WINDOW:	Fill();			break;
	case UPD_NEWLINE:	InsertNew();	break;
	case UPD_LINE:
	case UPD_ITEM:
	case UPD_DELITEM:	return;
	}
	SelectItem(Known);
}

void CArrayView::InsertNew() {
	POSITION pos = pDoc->Sketch.GetTailPosition();
	if (!pos) return;
	pLine = pDoc->Sketch.GetAt(pos);
	InsertLine (CurSel = nItems-1);
}

void CArrayView::Fill(){
	pList->DeleteAllItems();
	POSITION pos = pDoc->Sketch.GetHeadPosition();
	for (CurSel=0; pos; CurSel++) {
		pLine = pDoc->Sketch.GetNext(pos);
		InsertLine (CurSel);
	}
	if (CurSel) CurSel--;
}

void CArrayView::InsertLine (int n) {
	CString s;	s.Format("%d",n+1);
	pList->InsertItem (LVIF_TEXT,n,s,0,0,0,0L);
	SetLine(n);
}

void CArrayView::ChangeLine(TAttr attr) {
	switch (attr) {
	case WIDTH:  pLine->PenSize = PenSize;		break;
	case COLOR:  pLine->Color = Color;			break;
	case LENGTH: pLine->Points.SetSize(nPoints);	break;
	}
	SetLine(CurSel);
	pDoc->UpdateAllViews(this,UPD_LINE,pLine);
	pDoc->SetModifiedFlag();
	pLine->SetRect(pDoc->rMax);	//   
}

void CArrayView::Refresh() {
	nItems = pDoc->Sketch.GetCount();
	if (CurSel >= nItems && CurSel>0)
		CurSel--;
	for (int i=0; i<nItems; i++) {
		CString s;		s.Format("%d",i+1);
		pList->SetItemText (i,0,s);
	}
	SelectItem(Known);
}

void CArrayView::OnContextMenu(CWnd* pWnd, CPoint point) {
	if (!nItems) return;
	CMenu menu;
	menu.LoadMenu(IDR_RBUTTON);
	SelectItem(Find);
	menu.GetSubMenu(0)->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON,
			point.x, point.y, this);
}

void CArrayView::OnLineSize() {
	CAttrDlg(this,WIDTH).DoModal();
	ChangeLine(WIDTH);
}

void CArrayView::OnLineColor() {
	CColorDialog dlg(Color);
	if (dlg.DoModal()!=IDOK) return;
	Color = dlg.m_cc.rgbResult;
	ChangeLine(COLOR);
}

void CArrayView::OnLinePoints() {
	UINT nOld = nPoints;
	CAttrDlg(this,LENGTH).DoModal();
	if (nPoints>nOld) {
		MessageBox("You may only decrease the number","Input Error",MB_OK);
		nPoints = nOld;
		return;
	}
	ChangeLine(LENGTH);
}

void CArrayView::OnLineDel() {
	POSITION pos = SelectItem(Find);
	if (!pos)	return;
	delete pLine;
	pDoc->Sketch.RemoveAt(pos);
	pList->DeleteItem(CurSel);
	pDoc->UpdateAllViews(this, UPD_WINDOW);
	pDoc->SetModifiedFlag();
	Refresh();
}

void CArrayView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) {
	if (!nItems)	return;
	if (nChar==VK_DELETE)
		OnLineDel();
}

	
// CAttrDlg dialog

CAttrDlg::CAttrDlg(CWnd* p, TAttr attr)
	: CDialog(CAttrDlg::IDD, p) {
	pView = (CArrayView*)p;
	Attr = attr;
	//{{AFX_DATA_INIT(CAttrDlg)
	//}}AFX_DATA_INIT
}


void CAttrDlg::DoDataExchange(CDataExchange* pDX){
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAttrDlg)
	//}}AFX_DATA_MAP
	switch (Attr) {
	case WIDTH:
		DDX_Text(pDX, IDC_PEN, pView->PenSize);  
		DDV_MinMaxUInt(pDX, pView->PenSize, 1, 150);  break;
	case LENGTH:
		DDX_Text(pDX, IDC_PEN, pView->nPoints);  
		DDV_MinMaxUInt(pDX, pView->nPoints, 1, 150);
	}
}


BEGIN_MESSAGE_MAP(CAttrDlg, CDialog)
	//{{AFX_MSG_MAP(CAttrDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CAttrDlg::OnInitDialog() {
	CDialog::OnInitDialog();
	CString s;
	switch (Attr) {
	case WIDTH:  s = "Pen Size:";	break;
	case LENGTH: s = "Points:";		break;
	}
	SetDlgItemText(IDC_PROMPT, s);
	return TRUE; 
}
