#include "stdafx.h"
#include "Drawing.h"

#include "DrawingDoc.h"
#include "DrawingView.h"
#include "CntrItem.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_SERIAL(CDrawingCntrItem, COleClientItem, 0)

BEGIN_MESSAGE_MAP(CDrawingCntrItem, COleClientItem)
	//{{AFX_MSG_MAP(CDrawingCntrItem)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

CDrawingCntrItem::CDrawingCntrItem(CDrawingDoc* pContainer)
	: COleClientItem(pContainer), 
	rObj(10, 10, 300, 400),	szMax(600, 700)
{ }

CDrawingCntrItem::~CDrawingCntrItem(){}

void CDrawingCntrItem::OnGetItemPosition(CRect& rPosition) {
	// Get the coordinates of the item being edited in place (pixels)
	ASSERT_VALID(this);
	rPosition = rObj;
	CDrawingView* pView = GetActiveView();
	ASSERT_VALID(pView);
	CClientDC dc(0);
	pView->OnPrepareDC(&dc);
	dc.LPtoDP(rPosition);
}

void CDrawingCntrItem::OnChange(OLE_NOTIFICATION nCode, DWORD dwParam) {
	ASSERT_VALID(this);
	COleClientItem::OnChange(nCode, dwParam);
	switch (nCode) 	{
	case OLE_CHANGED:
		InvalidateItem();
		UpdateFromServerExtent();
		break;
	case OLE_CHANGED_STATE:
	case OLE_CHANGED_ASPECT:
		InvalidateItem();
		break;
	}
}

BOOL CDrawingCntrItem::OnChangeItemPosition(const CRect& rectPos) {
	CDrawingView* pView = GetActiveView();
	if ( !pView )	return FALSE;
	CRect r = rectPos;
	CClientDC dc(0);
	pView->OnPrepareDC(&dc);
	dc.DPtoLP(r);
	if (r != rObj)
		ChangeSelection(r);
	return COleClientItem::OnChangeItemPosition(rectPos);
}

void CDrawingCntrItem::InvalidateItem(){
	GetDocument()->UpdateAllViews(NULL, UPD_ITEM, this);
}

void CDrawingCntrItem::UpdateFromServerExtent() {
	CSize szReal, szObj = rObj.Size();
	if (!GetCachedExtent(&szReal))	// HIMETRIC
		return;
	CClientDC dc(NULL);
	dc.HIMETRICtoDP(&szReal);		// Pixels
	
	if ( (szReal.cx != szObj.cx || szReal.cy != szObj.cy)
			&&	!IsInPlaceActive() ) 
		ChangeSelection (CRect(rObj.TopLeft(),CSize (min(szReal.cx,szMax.cx),
								min(szReal.cy,szMax.cy))));
}

void CDrawingCntrItem::ChangeSelection(CRect r) {
	InvalidateItem();	// invalidate old item
	rObj = r;			// update to new rectangle
	InvalidateItem();	// invalidate new item
	if (IsInPlaceActive())
		SetItemRects();
	GetDocument()->SetModifiedFlag();
}

void CDrawingCntrItem::OnActivate() {
    CDrawingView* pView = GetActiveView();
    ASSERT_VALID(pView);
    COleClientItem* pItem = GetDocument()->GetInPlaceActiveItem(pView);
    if (pItem && pItem != this)
        pItem->Close();
    COleClientItem::OnActivate();
}

void CDrawingCntrItem::OnDeactivateUI(BOOL bUndoable) {
	COleClientItem::OnDeactivateUI(bUndoable);
    // Hide the object if it is not an outside-in object
    DWORD dwMisc = 0;
    m_lpObject->GetMiscStatus(GetDrawAspect(), &dwMisc);
    if (dwMisc & OLEMISC_INSIDEOUT)
        DoVerb(OLEIVERB_HIDE, NULL);
}

void CDrawingCntrItem::Serialize(CArchive& ar) {
	COleClientItem::Serialize(ar);
	if (ar.IsStoring())	{  ar << rObj << szMax;	}
	else				{  ar >> rObj >> szMax;	}
}

#ifdef _DEBUG
void CDrawingCntrItem::AssertValid() const {
	COleClientItem::AssertValid();
}

void CDrawingCntrItem::Dump(CDumpContext& dc) const {
	COleClientItem::Dump(dc);
}
#endif

COleDataSource* CDrawingCntrItem::OnGetClipboardData(
			BOOL bIncludeLink,LPPOINT lpOffset, LPSIZE lpSize)
{
	ASSERT_VALID(this);
								//   
	COleDataSource* pDS = new COleDataSource;
	try
	{
		GetNative(pDS);			//       
		GetClipboardData(pDS, bIncludeLink, lpOffset, lpSize);
	}
	catch(...)
	{
		delete pDS;
	}
	ASSERT_VALID(pDS);
	return pDS;
}

void CDrawingCntrItem::GetNative(COleDataSource* pDS)
{
	ASSERT_VALID(this);
	ASSERT_VALID(GetDocument());
	CSharedFile file;	//   
	CArchive ar(&file, CArchive::store);
	Serialize(ar);		//   OLE-
	ar.Close();
						//   
	pDS->CacheGlobalData(CDrawingDoc::cfClipb, file.Detach());
}
