#include "stdafx.h"
#include "Drawing.h"

#include "DrawingDoc.h"
#include "MainFrm.h"
#include "ArrayView.h"
#include "PenDlg.h"
#include "DrawingView.h"
#include "SrvrItem.h"
#include "CntrItem.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CLIPFORMAT CDrawingDoc::cfClipb = NULL;

IMPLEMENT_DYNCREATE(CDrawingDoc, COleServerDoc)

BEGIN_MESSAGE_MAP(CDrawingDoc, COleServerDoc)
	//{{AFX_MSG_MAP(CDrawingDoc)
	ON_COMMAND(ID_EDIT_CLEAR_ALL, OnEditClearAll)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CLEAR_ALL, OnUpdateEditClearAll)
	ON_COMMAND(ID_EDIT_COLOR, OnEditColor)
	ON_COMMAND(ID_EDIT_PEN, OnEditPen)
	//}}AFX_MSG_MAP
 	ON_UPDATE_COMMAND_UI(ID_OLE_VERB_FIRST,OnUpdateObjectVerbMenu)
 	ON_COMMAND(ID_OLE_EDIT_LINKS, OnEditLinks)
 	ON_COMMAND(ID_OLE_EDIT_CHANGE_ICON, OnEditChangeIcon)
 	ON_UPDATE_COMMAND_UI(ID_OLE_EDIT_LINKS,OnUpdateEditLinksMenu)
 	ON_UPDATE_COMMAND_UI(ID_OLE_EDIT_CHANGE_ICON,OnUpdateEditChangeIcon)
 	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdatePasteMenu)
 	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE_LINK,OnUpdatePasteLinkMenu)
END_MESSAGE_MAP()

CDrawingDoc::CDrawingDoc() : szDoc(800,900), szSrvr(300,400)
{
	PenSize = 2;	
	Color = RGB(0,0,255);
	bFirst = false;			//     
	bNeedUpdate = true;
	rMax.SetRectEmpty();	//  
	EnableCompoundFile();
	if ( !cfClipb )			//     
		cfClipb = (CLIPFORMAT)::RegisterClipboardFormat(_T("Drawing Document Format"));
}

CDrawingDoc::~CDrawingDoc() { }

BOOL CDrawingDoc::OnNewDocument()
{
	return COleServerDoc::OnNewDocument();
}

void CDrawingDoc::Serialize(CArchive& ar)
{
	Sketch.Serialize(ar);
	if (ar.IsStoring())	{ ar<<szDoc<<rMax; } 
	else				{ ar>>szDoc>>rMax; }
	COleDocument::Serialize(ar);
}

#ifdef _DEBUG
void CDrawingDoc::AssertValid() const {
	COleServerDoc::AssertValid();
}

void CDrawingDoc::Dump(CDumpContext& dc) const { COleServerDoc::Dump(dc); }
#endif

void CDrawingDoc::OnEditClearAll()
{
	bFirst = false;				//   
	rMax.SetRectEmpty();		//   
	DeleteSketch();				//  
								//      
	POSITION pos = GetStartPosition();
	while (pos)
	{
		CDrawingCntrItem* pItem = DYNAMIC_DOWNCAST(CDrawingCntrItem, GetNextClientItem(pos));
		//CDrawingCntrItem* pItem = (CDrawingCntrItem*)GetNextClientItem(pos);
		if (pItem)
			pItem->Delete();
	}
	SetModifiedFlag();
	UpdateAllViews(NULL);
	NotifyChanged();			//   
}

void CDrawingDoc::OnUpdateEditClearAll(CCmdUI* pCmdUI)
{										//    
	POSITION pos = GetStartPosition();	//   
	CDrawingCntrItem* pItem = (CDrawingCntrItem*)GetNextClientItem(pos);
	//  ,          
	pCmdUI->Enable (Sketch.GetCount() || pItem != NULL);
}

void CDrawingDoc::DeleteSketch()
{
	while (!Sketch.IsEmpty())
		delete Sketch.RemoveHead();
}

void CDrawingDoc::OnEditColor()
{
	CColorDialog dlg(Color);
	if (dlg.DoModal()!=IDOK) return;
	Color = dlg.m_cc.rgbResult;
}

void CDrawingDoc::OnEditPen()
{
	COLORREF r, g, b;
	r = GetRValue(Color);
	g = GetGValue(Color);
	b = GetBValue(Color);
	r = r ? RGB(r,0,0) : RGB(255,255,255);
	g = g ? RGB(0,g,0) : RGB(255,255,255);
	b = b ? RGB(0,0,b) : RGB(255,255,255);
	CPenDlg(this,r,g,b).DoModal();
}

COleServerItem* CDrawingDoc::OnGetEmbeddedItem()
{
	CDrawingSrvrItem* pItem = new CDrawingSrvrItem(this);
	ASSERT_VALID(pItem);
	return pItem;
}

CDrawingCntrItem* CDrawingDoc::CreateItem()
{
		//      
	return new CDrawingCntrItem(this);
}

void CDrawingDoc::DoPaste(CDocument *p)
{
	CDrawingDoc *pD = (CDrawingDoc *)p;
	if (pD==this) return;
	DeleteSketch();
	rMax = pD->rMax;			//   
	POSITION s = pD->Sketch.GetHeadPosition();
	while (s)
	{
		CLine *ps = pD->Sketch.GetNext(s);
		CLine *pt = new CLine(ps->PenSize,ps->Color);
		pt->rect = ps->rect;
		int size = ps->Points.GetSize();
		pt->Points.SetSize(size);
		for (int i=0; i<size; i++)
			pt->Points[i] = ps->Points[i];
		Sketch.AddTail(pt);
	}
	SetModifiedFlag();			//  
	UpdateAllViews(0);
	NotifyChanged();
}

void CDrawingDoc::Recalc(CPoint& pt)
{
	pt = rMax.CenterPoint() - pt;		//  
	rMax -= pt;							//   
										//     
	POSITION pos = Sketch.GetHeadPosition();
	while (pos)
	{									//   
		CLine *pLine = Sketch.GetNext(pos);
		pLine->Points[0] -= pt;			//   
		pLine->rect -= pt;				//   
	}
	SetModifiedFlag();
	UpdateAllViews(0);					//   
	NotifyChanged();
}

BOOL CDrawingDoc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if (!COleServerDoc::OnOpenDocument(lpszPathName))
		return FALSE;
	bFirst = true;
	return TRUE;
}

void CDrawingDoc::ShiftItem(CDrawingCntrItem* pItem)
{
	POSITION pos = GetStartPosition();

	while (pos)						//    
	{								//   
		CDrawingCntrItem* p = 
						(CDrawingCntrItem*)GetNextClientItem(pos);
		if (p->rObj == pItem->rObj && p != pItem)
			pItem->rObj.OffsetRect(10,10);		// 
	}
}

void CDrawingDoc::OnShowViews(BOOL bVisible)
{
	if (bVisible && bNeedUpdate)
	{
									//     
		COleUpdateDialog(this).DoModal();
		bNeedUpdate = false;
	}	
	COleServerDoc::OnShowViews(bVisible);
}
